package org.jastadd.ros2rag.compiler;

import beaver.Parser;
import org.jastadd.ros2rag.ast.*;
import org.jastadd.ros2rag.parser.Ros2RagParser;
import org.jastadd.ros2rag.scanner.Ros2RagScanner;

import java.io.BufferedReader;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;

/**
 * Testing Ros2Rag without parser.
 *
 * @author rschoene - Initial contribution
 */
public class SimpleMain {

  // --- just testing byte[] conversion ---
  public static void testing() {
    byte[] bytes;
    int i = 1;
    double d = 2.3d;
    float f = 4.2f;
    short sh = 13;
    long l = 7L;
    String s = "Hello";
    char c = 'a';

    Integer ii = Integer.valueOf(1);
    if (!ii.equals(i)) throw new AssertionError("Ints not equal");

    // int to byte
    ByteBuffer i2b = ByteBuffer.allocate(4);
    i2b.putInt(i);
    bytes = i2b.array();

    // byte to int
    ByteBuffer b2i = ByteBuffer.wrap(bytes);
    int actual_i = b2i.getInt();
    if (i != actual_i) throw new AssertionError("Ints not equal");

    // double to byte
    ByteBuffer d2b = ByteBuffer.allocate(8);
    d2b.putDouble(d);
    bytes = d2b.array();

    // byte to double
    ByteBuffer b2d = ByteBuffer.wrap(bytes);
    double actual_d = b2d.getDouble();
    if (d != actual_d) throw new AssertionError("Doubles not equal");

    // float to byte
    ByteBuffer f2b = ByteBuffer.allocate(4);
    f2b.putFloat(f);
    bytes = f2b.array();

    // byte to float
    ByteBuffer b2f = ByteBuffer.wrap(bytes);
    float actual_f = b2f.getFloat();
    if (f != actual_f) throw new AssertionError("Floats not equal");

    // short to byte
    ByteBuffer sh2b = ByteBuffer.allocate(2);
    sh2b.putShort(sh);
    bytes = sh2b.array();

    // byte to short
    ByteBuffer b2sh = ByteBuffer.wrap(bytes);
    short actual_sh = b2sh.getShort();
    if (sh != actual_sh) throw new AssertionError("Shorts not equal");

    // long to byte
    ByteBuffer l2b = ByteBuffer.allocate(8);
    l2b.putLong(l);
    bytes = l2b.array();

    // byte to long
    ByteBuffer b2l = ByteBuffer.wrap(bytes);
    long actual_l = b2l.getLong();
    if (l != actual_l) throw new AssertionError("Longs not equal");

    // String to byte
    bytes = s.getBytes();

    // byte to String
    String actual_s = new String(bytes);
    if (!s.equals(actual_s)) throw new AssertionError("Strings not equal");

    // char to byte
    ByteBuffer c2b = ByteBuffer.allocate(2);
    c2b.putChar(c);
    bytes = c2b.array();

    // byte to char
    ByteBuffer b2c = ByteBuffer.wrap(bytes);
    char actual_c = b2c.getChar();
    if (c != actual_c) throw new AssertionError("Floats not equal");
  }

  public static void main(String[] args) {
    testing();
//    createManualAST();
  }

  private static void createManualAST() {
    Ros2Rag model = new Ros2Rag();
    Program program = parseProgram(Paths.get("src", "test", "resources", "Example.relast"));
    model.setProgram(program);

    MappingDefinition mappingDefinition = new MappingDefinition();
    mappingDefinition.setID("PoseToPosition");
    mappingDefinition.setFromType(makeMappingDefinitionType("PBPose"));
    mappingDefinition.setFromVariableName("x");
    mappingDefinition.setToType(makeMappingDefinitionType("Position"));
    mappingDefinition.setContent("      pose.position.x += sqrt(.5 * size.x)\n" +
        "      MAP round(2)\n" +
        "      x = x / 100\n" +
        "      IGNORE_IF_SAME\n" +
        "    ;");
    model.addMappingDefinition(mappingDefinition);

    ReadFromMqttDefinition readFromMqttDefinition = new ReadFromMqttDefinition();
    readFromMqttDefinition.setAlwaysApply(false);
    readFromMqttDefinition.setToken(TokenComponent.createRef("Joint.CurrentPosition"));
    readFromMqttDefinition.addMapping(mappingDefinition);
    model.addUpdateDefinition(readFromMqttDefinition);

    model.treeResolveAll();

    System.out.println(model.generateAspect("Model"));
  }

  private static MappingDefinitionType makeMappingDefinitionType(String type) {
    JavaMappingDefinitionType result = new JavaMappingDefinitionType();
    result.setType(new SimpleJavaTypeUse(type));
    return result;
  }

  private static Program parseProgram(Path path) {
    try (BufferedReader reader = Files.newBufferedReader(path)) {
      Ros2RagScanner scanner = new Ros2RagScanner(reader);
      Ros2RagParser parser = new Ros2RagParser();
      return (Program) parser.parse(scanner);
    } catch (IOException | Parser.Exception e) {
      e.printStackTrace();
    }
    return null;
  }
}
